@@ -120,7 +120,7 @@ void bind_torrent_info()
         .def(init<boost::filesystem::path>())
         .def(init<boost::filesystem::wpath>())
 
-        .def("add_tracker", &torrent_info::add_tracker, (arg("url"), arg("tier")=0))
+        .def("add_tracker", &torrent_info::add_tracker, arg("url"))
         .def("add_url_seed", &torrent_info::add_url_seed)
 
         .def("name", &torrent_info::name, copy)
