@@ -57,76 +57,80 @@ int main(int argc, char* argv[])
 	boost::filesystem::path::default_name_check(boost::filesystem::no_check);
 #endif
 
-#ifndef BOOST_NO_EXCEPTIONS
-	try
+	int size = file_size(argv[1]);
+	if (size > 10 * 1000000)
 	{
-#endif
-
-		int size = file_size(argv[1]);
-		if (size > 10 * 1000000)
-		{
-			std::cerr << "file too big (" << size << "), aborting\n";
-			return 1;
-		}
-		std::vector<char> buf(size);
-		std::ifstream(argv[1], std::ios_base::binary).read(&buf[0], size);
-		lazy_entry e;
-		int ret = lazy_bdecode(&buf[0], &buf[0] + buf.size(), e);
-
-		if (ret != 0)
-		{
-			std::cerr << "invalid bencoding: " << ret << std::endl;
-			return 1;
-		}
-
-		std::cout << "\n\n----- raw info -----\n\n";
-		std::cout << e << std::endl;
-	
-		torrent_info t(e);
-
-		// print info about torrent
-		std::cout << "\n\n----- torrent file info -----\n\n";
-		std::cout << "nodes:\n";
-		typedef std::vector<std::pair<std::string, int> > node_vec;
-		node_vec const& nodes = t.nodes();
-		for (node_vec::const_iterator i = nodes.begin(), end(nodes.end());
-			i != end; ++i)
-		{
-			std::cout << i->first << ":" << i->second << "\n";
-		}
-		std::cout << "trackers:\n";
-		for (std::vector<announce_entry>::const_iterator i = t.trackers().begin();
-			i != t.trackers().end(); ++i)
-		{
-			std::cout << i->tier << ": " << i->url << "\n";
-		}
-
-		std::cout << "number of pieces: " << t.num_pieces() << "\n";
-		std::cout << "piece length: " << t.piece_length() << "\n";
-		std::cout << "info hash: " << t.info_hash() << "\n";
-		std::cout << "comment: " << t.comment() << "\n";
-		std::cout << "created by: " << t.creator() << "\n";
-		std::cout << "magnet link: " << make_magnet_uri(t) << "\n";
-		std::cout << "name: " << t.name() << "\n";
-		std::cout << "files:\n";
-		int index = 0;
-		for (torrent_info::file_iterator i = t.begin_files();
-			i != t.end_files(); ++i, ++index)
-		{
-			int first = t.map_file(index, 0, 0).piece;
-			int last = t.map_file(index, (std::max)(i->size - 1, size_type(0)), 1).piece;
-			std::cout << "  " << std::setw(11) << i->size
-				<< " " << i->path.string() << "[ " << first << ", "
-				<< last << " ]\n";
-		}
-
-#ifndef BOOST_NO_EXCEPTIONS
+		std::cerr << "file too big (" << size << "), aborting\n";
+		return 1;
 	}
-	catch (std::exception& e)
+	std::vector<char> buf(size);
+	std::ifstream(argv[1], std::ios_base::binary).read(&buf[0], size);
+	lazy_entry e;
+	int ret = lazy_bdecode(&buf[0], &buf[0] + buf.size(), e);
+
+	if (ret != 0)
 	{
-  		std::cout << e.what() << "\n";
+		std::cerr << "invalid bencoding: " << ret << std::endl;
+		return 1;
+	}
+
+	std::cout << "\n\n----- raw info -----\n\n";
+	std::cout << print_entry(e) << std::endl;
+
+	error_code ec;
+	torrent_info t(e, ec);
+	if (ec)
+	{
+		std::cout << ec.message() << std::endl;
+		return 1;
+	}
+
+	// print info about torrent
+	std::cout << "\n\n----- torrent file info -----\n\n";
+	std::cout << "nodes:\n";
+	typedef std::vector<std::pair<std::string, int> > node_vec;
+	node_vec const& nodes = t.nodes();
+	for (node_vec::const_iterator i = nodes.begin(), end(nodes.end());
+		i != end; ++i)
+	{
+		std::cout << i->first << ":" << i->second << "\n";
+	}
+	std::cout << "trackers:\n";
+	for (std::vector<announce_entry>::const_iterator i = t.trackers().begin();
+		i != t.trackers().end(); ++i)
+	{
+		std::cout << i->tier << ": " << i->url << "\n";
+	}
+
+	std::cout << "number of pieces: " << t.num_pieces() << "\n";
+	std::cout << "piece length: " << t.piece_length() << "\n";
+	char ih[41];
+	to_hex((char const*)&t.info_hash()[0], 20, ih);
+	std::cout << "info hash: " << ih << "\n";
+	std::cout << "comment: " << t.comment() << "\n";
+	std::cout << "created by: " << t.creator() << "\n";
+	std::cout << "magnet link: " << make_magnet_uri(t) << "\n";
+	std::cout << "name: " << t.name() << "\n";
+	std::cout << "files:\n";
+	int index = 0;
+	for (torrent_info::file_iterator i = t.begin_files();
+		i != t.end_files(); ++i, ++index)
+	{
+		int first = t.map_file(index, 0, 0).piece;
+		int last = t.map_file(index, (std::max)(i->size-1, size_type(0)), 0).piece;
+		std::cout << "  " << std::setw(11) << i->size
+			<< " "
+			<< (i->pad_file?'p':'-')
+			<< (i->executable_attribute?'x':'-')
+			<< (i->hidden_attribute?'h':'-')
+			<< (i->symlink_attribute?'l':'-')
+			<< " "
+			<< "[ " << std::setw(4) << first << ", " << std::setw(4) << last << " ]\t"
+			<< i->path.string() ;
+		if (i->symlink_attribute)
+			std::cout << " -> " << i->symlink_path.string();
+		std::cout << std::endl;
 	}
-#endif
 
 	return 0;
 }
