@@ -34,14 +34,17 @@ POSSIBILITY OF SUCH DAMAGE.
 #define TORRENT_DEBUG_HPP_INCLUDED
 
 #include <string>
+#include "libtorrent/config.hpp"
+
+#if TORRENT_USE_IOSTREAM
 #include <fstream>
 #include <iostream>
+#endif
 
 #ifdef _MSC_VER
 #pragma warning(push, 1)
 #endif
 
-#include <boost/lexical_cast.hpp>
 #include <boost/filesystem/fstream.hpp>
 #include <boost/filesystem/convenience.hpp>
 
@@ -60,11 +63,15 @@ namespace libtorrent
 	{
 		logger(fs::path const& logpath, fs::path const& filename, int instance, bool append = true)
 		{
+#if TORRENT_USE_IOSTREAM
+
 #ifndef BOOST_NO_EXCEPTIONS
 			try
 			{
 #endif
-				fs::path dir(fs::complete(logpath / ("libtorrent_logs" + boost::lexical_cast<std::string>(instance))));
+				char log_name[256];
+				snprintf(log_name, sizeof(log_name), "libtorrent_logs%d", instance);
+				fs::path dir(fs::complete(logpath / log_name));
 				if (!fs::exists(dir)) fs::create_directories(dir);
 				m_file.open((dir / filename).string().c_str(), std::ios_base::out | (append ? std::ios_base::app : std::ios_base::out));
 				*this << "\n\n\n*** starting log ***\n";
@@ -74,18 +81,23 @@ namespace libtorrent
 			{
 				std::cerr << "failed to create log '" << filename.string() << "': " << e.what() << std::endl;
 			}
+#endif
 #endif
 		}
 
 		template <class T>
 		logger& operator<<(T const& v)
 		{
+#if TORRENT_USE_IOSTREAM
 			m_file << v;
 			m_file.flush();
+#endif
 			return *this;
 		}
 
+#if TORRENT_USE_IOSTREAM
 		std::ofstream m_file;
+#endif
 	};
 
 }
