@@ -38,6 +38,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/config.hpp"
 #include "libtorrent/peer_id.hpp"
 #include "libtorrent/assert.hpp"
+#include "libtorrent/socket.hpp"
 
 namespace libtorrent { namespace dht
 {
@@ -57,6 +58,8 @@ int TORRENT_EXPORT distance_exp(node_id const& n1, node_id const& n2);
 
 node_id TORRENT_EXPORT generate_id();
 
+bool TORRENT_EXPORT verify_id(node_id const& nid, address const& source_ip);
+
 } } // namespace libtorrent::dht
 
 #endif // NODE_ID_HPP
