@@ -34,7 +34,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #define ROUTING_TABLE_HPP
 
 #include <vector>
-#include <deque>
 #include <boost/cstdint.hpp>
 
 #include <boost/iterator/iterator_facade.hpp>
@@ -52,6 +51,11 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <libtorrent/size_type.hpp>
 #include <libtorrent/assert.hpp>
 
+namespace libtorrent
+{
+	struct session_status;
+}
+
 namespace libtorrent { namespace dht
 {
 
@@ -151,7 +155,7 @@ namespace aux
 
 } // namespace aux
 
-class routing_table
+class TORRENT_EXPORT routing_table
 {
 public:
 	typedef aux::routing_table_iterator iterator;
@@ -160,6 +164,8 @@ public:
 	routing_table(node_id const& id, int bucket_size
 		, dht_settings const& settings);
 
+	void status(session_status& s) const;
+
 	void node_failed(node_id const& id);
 	
 	// adds an endpoint that will never be added to
@@ -183,15 +189,20 @@ public:
 	// the time from the last activity is more than 15 minutes
 	ptime next_refresh(int bucket);
 
+	enum
+	{
+		include_self = 1,
+		include_failed = 2
+	};
 	// fills the vector with the count nodes from our buckets that
 	// are nearest to the given id.
 	void find_node(node_id const& id, std::vector<node_entry>& l
-		, bool include_self, int count = 0);
+		, int options, int count = 0);
 	
-	// returns true if the given node would be placed in a bucket
-	// that is not full. If the node already exists in the table
-	// this function returns false
-	bool need_node(node_id const& id);
+	// this may add a node to the routing table and mark it as
+	// not pinged. If the bucket the node falls into is full,
+	// the node will be ignored.
+	void heard_about(node_id const& id, udp::endpoint const& ep);
 	
 	// this will set the given bucket's latest activity
 	// to the current time
@@ -237,6 +248,9 @@ private:
 	typedef boost::array<ptime, 160> table_activity_t;
 	table_activity_t m_bucket_activity;
 	node_id m_id; // our own node id
+
+	// the last time need_bootstrap() returned true
+	mutable ptime m_last_bootstrap;
 	
 	// this is a set of all the endpoints that have
 	// been identified as router nodes. They will
