@@ -49,7 +49,6 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/peer_id.hpp"
 #include "libtorrent/socket.hpp"
-#include "libtorrent/session_settings.hpp"
 #include "libtorrent/config.hpp"
 #include "libtorrent/assert.hpp"
 #include "libtorrent/time.hpp"
@@ -63,6 +62,8 @@ namespace libtorrent
 
 	struct TORRENT_EXPORT piece_block
 	{
+		const static piece_block invalid;
+
 		piece_block() {}
 		piece_block(int p_index, int b_index)
 			: piece_index(p_index)
@@ -140,9 +141,10 @@ namespace libtorrent
 
 		struct downloading_piece
 		{
-			downloading_piece(): last_request(min_time()), finished(0), writing(0), requested(0) {}
+			downloading_piece(): state(none), index(-1), info(0)
+				, finished(0), writing(0), requested(0) {}
+
 			piece_state_t state;
-			ptime last_request;
 
 			// the index of the piece
 			int index;
@@ -276,11 +278,13 @@ namespace libtorrent
 		// marks this piece-block as queued for downloading
 		bool mark_as_downloading(piece_block block, void* peer
 			, piece_state_t s);
-		void mark_as_writing(piece_block block, void* peer);
+		// returns true if the block was marked as writing,
+		// and false if the block is already finished or writing
+		bool mark_as_writing(piece_block block, void* peer);
+
 		void mark_as_finished(piece_block block, void* peer);
 		void write_failed(piece_block block);
 		int num_peers(piece_block block) const;
-		ptime last_request(int piece) const;
 
 		// returns information about the given piece
 		void piece_info(int index, piece_picker::downloading_piece& st) const;
@@ -543,17 +547,6 @@ namespace libtorrent
 		enum { max_pieces = piece_pos::we_have_index - 1 };
 
 	};
-
-	inline int piece_picker::blocks_in_piece(int index) const
-	{
-		TORRENT_ASSERT(index >= 0);
-		TORRENT_ASSERT(index < (int)m_piece_map.size());
-		if (index+1 == (int)m_piece_map.size())
-			return m_blocks_in_last_piece;
-		else
-			return m_blocks_per_piece;
-	}
-
 }
 
 #endif // TORRENT_PIECE_PICKER_HPP_INCLUDED
