@@ -61,8 +61,13 @@ namespace libtorrent
 
 		void operator+=(stat_channel const& s)
 		{
+			TORRENT_ASSERT(m_counter >= 0);
+			TORRENT_ASSERT(m_total_counter >= 0);
+			TORRENT_ASSERT(s.m_counter >= 0);
 			m_counter += s.m_counter;
 			m_total_counter += s.m_counter;
+			TORRENT_ASSERT(m_counter >= 0);
+			TORRENT_ASSERT(m_total_counter >= 0);
 		}
 
 		void add(int count)
@@ -70,19 +75,23 @@ namespace libtorrent
 			TORRENT_ASSERT(count >= 0);
 
 			m_counter += count;
+			TORRENT_ASSERT(m_counter >= 0);
 			m_total_counter += count;
+			TORRENT_ASSERT(m_total_counter >= 0);
 		}
 
 		// should be called once every second
-		void second_tick(float tick_interval);
-		float rate() const { return m_rate_sum / float(history); }
+		void second_tick(int tick_interval_ms);
+		int rate() const { return m_rate_sum / history; }
 		size_type rate_sum() const { return m_rate_sum; }
 		size_type total() const { return m_total_counter; }
 
-		void offset(size_type counter)
+		void offset(size_type c)
 		{
-			TORRENT_ASSERT(counter >= 0);
-			m_total_counter += counter;
+			TORRENT_ASSERT(c >= 0);
+			TORRENT_ASSERT(m_total_counter >= 0);
+			m_total_counter += c;
+			TORRENT_ASSERT(m_total_counter >= 0);
 		}
 
 		size_type counter() const { return m_counter; }
@@ -100,7 +109,7 @@ namespace libtorrent
 #ifdef TORRENT_DEBUG
 		void check_invariant() const
 		{
-			int sum = 0;
+			size_type sum = 0;
 			for (int i = 0; i < history; ++i) sum += m_rate_history[i];
 			TORRENT_ASSERT(m_rate_sum == sum);
 			TORRENT_ASSERT(m_total_counter >= 0);
@@ -130,6 +139,42 @@ namespace libtorrent
 				m_stat[i] += s.m_stat[i];
 		}
 
+		void sent_syn(bool ipv6)
+		{
+			m_stat[upload_ip_protocol].add(ipv6 ? 60 : 40);
+		}
+
+		void received_synack(bool ipv6)
+		{
+			// we received SYN-ACK and also sent ACK back
+			m_stat[download_ip_protocol].add(ipv6 ? 60 : 40);
+			m_stat[upload_ip_protocol].add(ipv6 ? 60 : 40);
+		}
+
+		void received_dht_bytes(int bytes)
+		{
+			TORRENT_ASSERT(bytes >= 0);
+			m_stat[download_dht_protocol].add(bytes);
+		}
+
+		void sent_dht_bytes(int bytes)
+		{
+			TORRENT_ASSERT(bytes >= 0);
+			m_stat[upload_dht_protocol].add(bytes);
+		}
+
+		void received_tracker_bytes(int bytes)
+		{
+			TORRENT_ASSERT(bytes >= 0);
+			m_stat[download_tracker_protocol].add(bytes);
+		}
+
+		void sent_tracker_bytes(int bytes)
+		{
+			TORRENT_ASSERT(bytes >= 0);
+			m_stat[upload_tracker_protocol].add(bytes);
+		}
+
 		void received_bytes(int bytes_payload, int bytes_protocol)
 		{
 			TORRENT_ASSERT(bytes_payload >= 0);
@@ -148,56 +193,75 @@ namespace libtorrent
 			m_stat[upload_protocol].add(bytes_protocol);
 		}
 
-		// calculate ip protocol overhead
-		void calc_ip_overhead()
+		// and IP packet was received or sent
+		// account for the overhead caused by it
+		void trancieve_ip_packet(int bytes_transferred, bool ipv6)
 		{
-			int uploaded = m_stat[upload_protocol].counter()
-				+ m_stat[upload_payload].counter();
-			int downloaded = m_stat[download_protocol].counter()
-				+ m_stat[download_payload].counter();
-
-			// IP + TCP headers are 40 bytes per MTU (1460)
-			// bytes of payload, but at least 40 bytes
-			m_stat[upload_ip_protocol].add((std::max)(uploaded / 1460, uploaded>0?40:0));
-			m_stat[download_ip_protocol].add((std::max)(downloaded / 1460, downloaded>0?40:0));
-
-			// also account for ACK traffic. That adds to the transfers
-			// in the opposite direction. Even on connections with symmetric
-			// transfer rates, it seems to add a penalty.
-			m_stat[upload_ip_protocol].add((std::max)(downloaded * 40 / 1460, downloaded>0?40:0));
-			m_stat[download_ip_protocol].add((std::max)(uploaded * 40 / 1460, uploaded>0?40:0));
+			// one TCP/IP packet header for the packet
+			// sent or received, and one for the ACK
+			// The IPv4 header is 20 bytes
+			// and IPv6 header is 40 bytes
+			const int header = (ipv6 ? 40 : 20) + 20;
+			const int mtu = 1500;
+			const int packet_size = mtu - header;
+			const int overhead = (std::max)(1, (bytes_transferred + packet_size - 1) / packet_size) * header;
+			m_stat[download_ip_protocol].add(overhead);
+			m_stat[upload_ip_protocol].add(overhead);
 		}
 
 		int upload_ip_overhead() const { return m_stat[upload_ip_protocol].counter(); }
 		int download_ip_overhead() const { return m_stat[download_ip_protocol].counter(); }
+		int upload_dht() const { return m_stat[upload_dht_protocol].counter(); }
+		int download_dht() const { return m_stat[download_dht_protocol].counter(); }
+		int download_tracker() const { return m_stat[download_tracker_protocol].counter(); }
+		int upload_tracker() const { return m_stat[upload_tracker_protocol].counter(); }
 
 		// should be called once every second
-		void second_tick(float tick_interval)
+		void second_tick(int tick_interval_ms)
 		{
 			for (int i = 0; i < num_channels; ++i)
-				m_stat[i].second_tick(tick_interval);
+				m_stat[i].second_tick(tick_interval_ms);
 		}
 
-		float upload_rate() const
+		int upload_rate() const
 		{
 			return (m_stat[upload_payload].rate_sum()
 				+ m_stat[upload_protocol].rate_sum()
-				+ m_stat[upload_ip_protocol].rate_sum())
-				/ float(stat_channel::history);
+				+ m_stat[upload_ip_protocol].rate_sum()
+				+ m_stat[upload_dht_protocol].rate_sum())
+				/ stat_channel::history;
 		}
 
-		float download_rate() const
+		int download_rate() const
 		{
 			return (m_stat[download_payload].rate_sum()
 				+ m_stat[download_protocol].rate_sum()
-				+ m_stat[download_ip_protocol].rate_sum())
-				/ float(stat_channel::history);
+				+ m_stat[download_ip_protocol].rate_sum()
+				+ m_stat[download_dht_protocol].rate_sum())
+				/ stat_channel::history;
 		}
 
-		float upload_payload_rate() const
-		{ return m_stat[upload_payload].rate(); }
+		size_type total_upload() const
+		{
+			return m_stat[upload_payload].total()
+				+ m_stat[upload_protocol].total()
+				+ m_stat[upload_ip_protocol].total()
+				+ m_stat[upload_dht_protocol].total()
+				+ m_stat[upload_tracker_protocol].total();
+		}
 
-		float download_payload_rate() const
+		size_type total_download() const
+		{
+			return m_stat[download_payload].total()
+				+ m_stat[download_protocol].total()
+				+ m_stat[download_ip_protocol].total()
+				+ m_stat[download_dht_protocol].total()
+				+ m_stat[download_tracker_protocol].total();
+		}
+
+		int upload_payload_rate() const
+		{ return m_stat[upload_payload].rate(); }
+		int download_payload_rate() const
 		{ return m_stat[download_payload].rate(); }
 
 		size_type total_payload_upload() const
@@ -210,13 +274,16 @@ namespace libtorrent
 		size_type total_protocol_download() const
 		{ return m_stat[download_protocol].total(); }
 
+		size_type total_transfer(int channel) const
+		{ return m_stat[channel].total(); }
+		int transfer_rate(int channel) const
+		{ return m_stat[channel].rate(); }
+
 		// this is used to offset the statistics when a
 		// peer_connection is opened and have some previous
 		// transfers from earlier connections.
 		void add_stat(size_type downloaded, size_type uploaded)
 		{
-			TORRENT_ASSERT(downloaded >= 0);
-			TORRENT_ASSERT(uploaded >= 0);
 			m_stat[download_payload].offset(downloaded);
 			m_stat[upload_payload].offset(uploaded);
 		}
@@ -225,14 +292,10 @@ namespace libtorrent
 		{ return m_stat[download_payload].counter(); }
 		size_type last_payload_uploaded() const
 		{ return m_stat[upload_payload].counter(); }
-
-		void clear()
-		{
-			for (int i = 0; i < num_channels; ++i)
-				m_stat[i].clear();
-		}
-
-	private:
+		size_type last_protocol_downloaded() const
+		{ return m_stat[download_protocol].counter(); }
+		size_type last_protocol_uploaded() const
+		{ return m_stat[upload_protocol].counter(); }
 
 		// these are the channels we keep stats for
 		enum
@@ -240,12 +303,30 @@ namespace libtorrent
 			upload_payload,
 			upload_protocol,
 			upload_ip_protocol,
+			upload_dht_protocol,
+			upload_tracker_protocol,
 			download_payload,
 			download_protocol,
 			download_ip_protocol,
+			download_dht_protocol,
+			download_tracker_protocol,
 			num_channels
 		};
 
+		void clear()
+		{
+			for (int i = 0; i < num_channels; ++i)
+				m_stat[i].clear();
+		}
+
+		stat_channel const& operator[](int i) const
+		{
+			TORRENT_ASSERT(i >= 0 && i < num_channels);
+			return m_stat[i];
+		}
+
+	private:
+
 		stat_channel m_stat[num_channels];
 	};
 
