@@ -74,6 +74,7 @@ namespace libtorrent
 	struct file_pool;
 	struct disk_io_job;
 	struct disk_buffer_pool;
+	struct session_settings;
 
 	enum storage_mode_t
 	{
@@ -298,7 +299,6 @@ namespace libtorrent
 		error_code const& error() const { return m_storage->error(); }
 		std::string const& error_file() const { return m_storage->error_file(); }
 		int last_piece() const { return m_last_piece; }
-		int last_operation() const { return m_last_op; }
 		void clear_error() { m_storage->clear_error(); }
 
 		int slot_for(int piece) const;
@@ -434,9 +434,6 @@ namespace libtorrent
 		// the last piece we wrote to or read from
 		int m_last_piece;
 
-		// the last operation we did (read or write)
-		int m_last_op;
-
 		// this is saved in case we need to instantiate a new
 		// storage (osed when remapping files)
 		storage_constructor_type m_storage_constructor;
