@@ -181,7 +181,7 @@ namespace libtorrent
 	typedef libtorrent_exception invalid_torrent_file;
 #endif
 
-	int TORRENT_EXPORT load_file(fs::path const& filename, std::vector<char>& v);
+	int TORRENT_EXPORT load_file(fs::path const& filename, std::vector<char>& v, error_code& ec);
 
 	class TORRENT_EXPORT torrent_info : public intrusive_ptr_base<torrent_info>
 	{
