@@ -63,7 +63,7 @@ namespace libtorrent
 			char const* val_start = 0;
 			int token;
 			// look for tag start
-			for(; *p != '<' && p != end; ++p);
+			for(; p != end && *p != '<'; ++p);
 
 			if (p != start)
 			{
@@ -83,7 +83,7 @@ namespace libtorrent
 			++p;	
 
 			// parse the name of the tag.
-			for (start = p; p != end && *p != '>' && !isspace(*p); ++p);
+			for (start = p; p != end && *p != '>' && !is_space(*p); ++p);
 
 			char* tag_name_end = p;
 
@@ -150,11 +150,11 @@ namespace libtorrent
 			for (char* i = tag_name_end; i < tag_end; ++i)
 			{
 				// find start of attribute name
-				for (; i != tag_end && isspace(*i); ++i);
+				for (; i != tag_end && is_space(*i); ++i);
 				if (i == tag_end) break;
 				start = i;
 				// find end of attribute name
-				for (; i != tag_end && *i != '=' && !isspace(*i); ++i);
+				for (; i != tag_end && *i != '=' && !is_space(*i); ++i);
 				char* name_end = i;
 
 				// look for equality sign
@@ -170,7 +170,7 @@ namespace libtorrent
 				}
 
 				++i;
-				for (; i != tag_end && isspace(*i); ++i);
+				for (; i != tag_end && is_space(*i); ++i);
 				// check for parse error (values must be quoted)
 				if (i == tag_end || (*i != '\'' && *i != '\"'))
 				{
