@@ -33,7 +33,10 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/pch.hpp"
 
 #include "libtorrent/alert.hpp"
+#include "libtorrent/alert_types.hpp"
 #include <boost/thread/xtime.hpp>
+#include <boost/function.hpp>
+#include <boost/bind.hpp>
 
 namespace libtorrent {
 
@@ -41,9 +44,10 @@ namespace libtorrent {
 	alert::~alert() {}
 	ptime alert::timestamp() const { return m_timestamp; }
 
-	alert_manager::alert_manager()
+	alert_manager::alert_manager(io_service& ios)
 		: m_alert_mask(alert::error_notification)
 		, m_queue_size_limit(queue_size_limit_default)
+		, m_ios(ios)
 	{}
 
 	alert_manager::~alert_manager()
@@ -75,15 +79,48 @@ namespace libtorrent {
 		xt.nsec = boost::xtime::xtime_nsec_t(nsec);
 		// apparently this call can be interrupted
 		// prematurely if there are other signals
-		if (!m_condition.timed_wait(lock, xt)) return 0;
-		if (m_alerts.empty()) return 0;
-		return m_alerts.front();
+		while (m_condition.timed_wait(lock, xt))
+			if (!m_alerts.empty()) return m_alerts.front();
+
+		return 0;
+	}
+
+	void alert_manager::set_dispatch_function(boost::function<void(alert const&)> const& fun)
+	{
+		boost::mutex::scoped_lock lock(m_mutex);
+
+		m_dispatch = fun;
+
+		std::queue<alert*> alerts = m_alerts;
+		while (!m_alerts.empty()) m_alerts.pop();
+		lock.unlock();
+
+		while (!alerts.empty())
+		{
+			m_dispatch(*alerts.front());
+			delete alerts.front();
+			alerts.pop();
+		}
+	}
+
+	void dispatch_alert(boost::function<void(alert const&)> dispatcher
+		, alert* alert_)
+	{
+		std::auto_ptr<alert> holder(alert_);
+		dispatcher(*alert_);
 	}
 
 	void alert_manager::post_alert(const alert& alert_)
 	{
 		boost::mutex::scoped_lock lock(m_mutex);
 
+		if (m_dispatch)
+		{
+			TORRENT_ASSERT(m_alerts.empty());
+			m_ios.post(boost::bind(&dispatch_alert, m_dispatch, alert_.clone().release()));
+			return;
+		}
+
 		if (m_alerts.size() >= m_queue_size_limit) return;
 		m_alerts.push(alert_.clone().release());
 		m_condition.notify_all();
@@ -115,5 +152,35 @@ namespace libtorrent {
 		return queue_size_limit_;
 	}
 
+	stats_alert::stats_alert(torrent_handle const& h, int in
+		, stat const& s)
+		: torrent_alert(h)
+		, interval(in)
+	{
+		for (int i = 0; i < num_channels; ++i)
+			transferred[i] = s[i].counter();
+	}
+
+	std::string stats_alert::message() const
+	{
+		char msg[200];
+		snprintf(msg, sizeof(msg), "%s: [%d] %d %d %d %d %d %d %d %d %d %d"
+			, torrent_alert::message().c_str()
+			, interval
+			, transferred[0]
+			, transferred[1]
+			, transferred[2]
+			, transferred[3]
+			, transferred[4]
+			, transferred[5]
+			, transferred[6]
+			, transferred[7]
+			, transferred[8]
+			, transferred[9]);
+		return msg;
+	}
+
+	cache_flushed_alert::cache_flushed_alert(torrent_handle const& h): torrent_alert(h) {}
+
 } // namespace libtorrent
 
