@@ -32,6 +32,8 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #ifdef TORRENT_DEBUG
 
+#include "libtorrent/config.hpp"
+
 #ifdef __APPLE__
 #include <AvailabilityMacros.h>
 #endif
@@ -116,7 +118,7 @@ void print_backtrace(char const* label) {}
 
 #endif
 
-void assert_fail(char const* expr, int line, char const* file, char const* function)
+TORRENT_EXPORT void assert_fail(char const* expr, int line, char const* file, char const* function)
 {
 
 	fprintf(stderr, "assertion failed. Please file a bugreport at "
