@@ -298,9 +298,9 @@ namespace libtorrent
 		, m_file_pool(fp)
 		, m_disk_io_thread(boost::ref(*this))
 	{
-#ifdef TORRENT_DISK_STATS
-		m_log.open("disk_io_thread.log", std::ios::trunc);
-#endif
+		// don't do anything in here. Essentially all members
+		// of this object are owned by the newly created thread.
+		// initialize stuff in operator().
 	}
 
 	disk_io_thread::~disk_io_thread()
@@ -721,14 +721,17 @@ namespace libtorrent
 			int block_size = (std::min)(piece_size - i * m_block_size, m_block_size);
 			TORRENT_ASSERT(offset + block_size <= piece_size);
 			TORRENT_ASSERT(offset + block_size > 0);
-			if (!buf)
+			if (iov)
 			{
+				TORRENT_ASSERT(!buf);
 				iov[iov_counter].iov_base = p.blocks[i].buf;
 				iov[iov_counter].iov_len = block_size;
 				++iov_counter;
 			}
 			else
 			{
+				TORRENT_ASSERT(buf);
+				TORRENT_ASSERT(iov == 0);
 				std::memcpy(buf.get() + offset, p.blocks[i].buf, block_size);
 				offset += m_block_size;
 			}
@@ -907,7 +910,7 @@ namespace libtorrent
 			{
 				TORRENT_ASSERT(iov[i].iov_base);
 				TORRENT_ASSERT(iov[i].iov_len > 0);
-				TORRENT_ASSERT(offset + iov[i].iov_len <= buffer_size);
+				TORRENT_ASSERT(int(offset + iov[i].iov_len) <= buffer_size);
 				std::memcpy(iov[i].iov_base, buf.get() + offset, iov[i].iov_len);
 				offset += iov[i].iov_len;
 			}
@@ -1364,24 +1367,27 @@ namespace libtorrent
 
 	bool should_cancel_on_abort(disk_io_job const& j)
 	{
-		TORRENT_ASSERT(j.action >= 0 && j.action < sizeof(action_flags));
+		TORRENT_ASSERT(j.action >= 0 && j.action < int(sizeof(action_flags)));
 		return action_flags[j.action] & cancel_on_abort;
 	}
 
 	bool is_read_operation(disk_io_job const& j)
 	{
-		TORRENT_ASSERT(j.action >= 0 && j.action < sizeof(action_flags));
+		TORRENT_ASSERT(j.action >= 0 && j.action < int(sizeof(action_flags)));
 		return action_flags[j.action] & read_operation;
 	}
 
 	bool operation_has_buffer(disk_io_job const& j)
 	{
-		TORRENT_ASSERT(j.action >= 0 && j.action < sizeof(action_flags));
+		TORRENT_ASSERT(j.action >= 0 && j.action < int(sizeof(action_flags)));
 		return action_flags[j.action] & buffer_operation;
 	}
 
 	void disk_io_thread::operator()()
 	{
+#ifdef TORRENT_DISK_STATS
+		m_log.open("disk_io_thread.log", std::ios::trunc);
+#endif
 		// 1 = forward in list, -1 = backwards in list
 		int elevator_direction = 1;
 
@@ -2029,22 +2035,30 @@ namespace libtorrent
 					mutex_t::scoped_lock l(m_piece_mutex);
 					INVARIANT_CHECK;
 
-					cache_t::iterator i = std::remove_if(
-						m_pieces.begin(), m_pieces.end(), boost::bind(&cached_piece_entry::storage, _1) == j.storage);
-
-					for (cache_t::iterator k = i; k != m_pieces.end(); ++k)
+					// delete all write cache entries for this storage
+					torrent_info const& ti = *j.storage->info();
+					for (cache_t::iterator i = m_pieces.begin(); i != m_pieces.end();)
 					{
-						torrent_info const& ti = *k->storage->info();
-						int blocks_in_piece = (ti.piece_size(k->piece) + m_block_size - 1) / m_block_size;
+						cached_piece_entry& e = *i;
+						if (e.storage != j.storage)
+						{
+							++i;
+							continue;
+						}
+
+						int blocks_in_piece = (ti.piece_size(e.piece) + m_block_size - 1) / m_block_size;
 						for (int j = 0; j < blocks_in_piece; ++j)
 						{
-							if (k->blocks[j].buf == 0) continue;
-							free_buffer(k->blocks[j].buf);
-							k->blocks[j].buf = 0;
+							if (e.blocks[j].buf == 0) continue;
+							free_buffer(e.blocks[j].buf);
+							e.blocks[j].buf = 0;
 							--m_cache_stats.cache_size;
+							TORRENT_ASSERT(e.num_blocks > 0);
+							--e.num_blocks;
 						}
+						TORRENT_ASSERT(e.num_blocks == 0);
+						m_pieces.erase(i++);
 					}
-					m_pieces.erase(i, m_pieces.end());
 					l.unlock();
 					release_memory();
 
@@ -2060,6 +2074,7 @@ namespace libtorrent
 					lazy_entry const* rd = (lazy_entry const*)j.buffer;
 					TORRENT_ASSERT(rd != 0);
 					ret = j.storage->check_fastresume(*rd, j.error);
+					test_error(j);
 					break;
 				}
 				case disk_io_job::check_files:
