@@ -74,7 +74,7 @@ namespace libtorrent
 
 		// send CONNECT
 		std::back_insert_iterator<std::vector<char> > p(m_buffer);
-		write_string("CONNECT " + boost::lexical_cast<std::string>(m_remote_endpoint)
+		write_string("CONNECT " + print_endpoint(m_remote_endpoint)
 			+ " HTTP/1.0\r\n", p);
 		if (!m_user.empty())
 		{
