@@ -34,7 +34,6 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/ip_filter.hpp"
 #include <boost/utility.hpp>
-//#include <iostream>
 
 
 namespace libtorrent
@@ -46,11 +45,13 @@ namespace libtorrent
 			TORRENT_ASSERT(last.is_v4());
 			m_filter4.add_rule(first.to_v4().to_bytes(), last.to_v4().to_bytes(), flags);
 		}
+#if TORRENT_USE_IPV6
 		else if (first.is_v6())
 		{
 			TORRENT_ASSERT(last.is_v6());
 			m_filter6.add_rule(first.to_v6().to_bytes(), last.to_v6().to_bytes(), flags);
 		}
+#endif
 		else
 			TORRENT_ASSERT(false);
 	}
@@ -59,14 +60,23 @@ namespace libtorrent
 	{
 		if (addr.is_v4())
 			return m_filter4.access(addr.to_v4().to_bytes());
+#if TORRENT_USE_IPV6
 		TORRENT_ASSERT(addr.is_v6());
 		return m_filter6.access(addr.to_v6().to_bytes());
+#else
+		return 0;
+#endif
 	}
 
 	ip_filter::filter_tuple_t ip_filter::export_filter() const
 	{
+#if TORRENT_USE_IPV6
 		return boost::make_tuple(m_filter4.export_filter<address_v4>()
 			, m_filter6.export_filter<address_v6>());
+#else
+		return boost::make_tuple(m_filter4.export_filter<address_v4>()
+			, std::vector<ip_range<address_v6> >());
+#endif
 	}
 	
 	void port_filter::add_rule(boost::uint16_t first, boost::uint16_t last, int flags)
