@@ -35,6 +35,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <libtorrent/kademlia/find_data.hpp>
 #include <libtorrent/kademlia/routing_table.hpp>
 #include <libtorrent/kademlia/rpc_manager.hpp>
+#include <libtorrent/kademlia/node.hpp>
 #include <libtorrent/io.hpp>
 #include <libtorrent/socket.hpp>
 
@@ -54,16 +55,18 @@ void find_data_observer::reply(msg const& m)
 		return;
 	}
 
+	if (!m.write_token.empty())
+		m_algorithm->got_write_token(m.id, m.write_token);
+
 	if (!m.peers.empty())
-	{
 		m_algorithm->got_data(&m);
-	}
-	else
+
+	if (!m.nodes.empty())
 	{
 		for (msg::nodes_t::const_iterator i = m.nodes.begin()
 			, end(m.nodes.end()); i != end; ++i)
 		{
-			m_algorithm->traverse(i->id, i->addr);	
+			m_algorithm->traverse(i->id, udp::endpoint(i->addr, i->port));
 		}
 	}
 	m_algorithm->finished(m_self);
@@ -79,23 +82,14 @@ void find_data_observer::timeout()
 
 
 find_data::find_data(
-	node_id target
-	, int branch_factor
-	, int max_results
-	, routing_table& table
-	, rpc_manager& rpc
-	, done_callback const& callback
-)
-	: traversal_algorithm(
-		target
-		, branch_factor
-		, max_results
-		, table
-		, rpc
-		, table.begin()
-		, table.end()
-	)
-	, m_done_callback(callback)
+	node_impl& node
+	, node_id target
+	, data_callback const& dcallback
+	, nodes_callback const& ncallback)
+	: traversal_algorithm(node, target, node.m_table.begin(), node.m_table.end())
+	, m_data_callback(dcallback)
+	, m_nodes_callback(ncallback)
+	, m_target(target)
 	, m_done(false)
 {
 	boost::intrusive_ptr<find_data> self(this);
@@ -110,37 +104,43 @@ void find_data::invoke(node_id const& id, udp::endpoint addr)
 		return;
 	}
 
-	TORRENT_ASSERT(m_rpc.allocation_size() >= sizeof(find_data_observer));
-	observer_ptr o(new (m_rpc.allocator().malloc()) find_data_observer(this, id, m_target));
-#ifndef NDEBUG
+	TORRENT_ASSERT(m_node.m_rpc.allocation_size() >= sizeof(find_data_observer));
+	void* ptr = m_node.m_rpc.allocator().malloc();
+	if (ptr == 0)
+	{
+		done();
+		return;
+	}
+	m_node.m_rpc.allocator().set_next_size(10);
+	observer_ptr o(new (ptr) find_data_observer(this, id));
+#ifdef TORRENT_DEBUG
 	o->m_in_constructor = false;
 #endif
-	m_rpc.invoke(messages::get_peers, addr, o);
+	m_node.m_rpc.invoke(messages::get_peers, addr, o);
 }
 
 void find_data::got_data(msg const* m)
 {
-	m_done = true;
-	m_done_callback(m);
+	m_data_callback(m->peers);
 }
 
 void find_data::done()
 {
 	if (m_invoke_count != 0) return;
-	if (!m_done) m_done_callback(0);
-}
 
-void find_data::initiate(
-	node_id target
-	, int branch_factor
-	, int max_results
-	, routing_table& table
-	, rpc_manager& rpc
-	, done_callback const& callback
-)
-{
-	std::cerr << "find_data::initiate, key: " << target << "\n";
-	new find_data(target, branch_factor, max_results, table, rpc, callback);
+	std::vector<std::pair<node_entry, std::string> > results;
+	int num_results = m_node.m_table.bucket_size();
+	for (std::vector<result>::iterator i = m_results.begin()
+		, end(m_results.end()); i != end && num_results > 0; ++i)
+	{
+		if (i->flags & result::no_id) continue;
+		if ((i->flags & result::queried) == 0) continue;
+		std::map<node_id, std::string>::iterator j = m_write_tokens.find(i->id);
+		if (j == m_write_tokens.end()) continue;
+		results.push_back(std::make_pair(node_entry(i->id, i->addr), j->second));
+		--num_results;
+	}
+	m_nodes_callback(results);
 }
 
 } } // namespace libtorrent::dht
