@@ -221,7 +221,7 @@ bool rpc_manager::incoming(msg const& m)
 			reply.message_id = messages::error;
 			reply.error_code = 203; // Protocol error
 			char msg[100];
-			snprintf(msg, sizeof(msg), "reply with invaliud transaction "
+			snprintf(msg, sizeof(msg), "reply with invalid transaction "
 				"id, size %d", int(m.transaction_id.size()));
 			reply.addr = m.addr;
 			reply.transaction_id = "";
