@@ -52,7 +52,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <cstdlib>
 #include <boost/config.hpp>
 
-using boost::bind;
 using namespace libtorrent;
 
 namespace libtorrent
@@ -68,7 +67,7 @@ lsd::lsd(io_service& ios, address const& listen_interface
 	: m_callback(cb)
 	, m_retry_count(1)
 	, m_socket(ios, udp::endpoint(address_v4::from_string("239.192.152.143", ec), 6771)
-		, bind(&lsd::on_announce, self(), _1, _2, _3))
+		, boost::bind(&lsd::on_announce, self(), _1, _2, _3))
 	, m_broadcast_timer(ios)
 	, m_disabled(false)
 {
@@ -213,3 +212,8 @@ void lsd::close()
 	m_callback.clear();
 }
 
+void lsd::use_broadcast(bool b)
+{
+	m_socket.enable_ip_broadcast(b);
+}
+
