@@ -396,7 +396,8 @@ namespace libtorrent
 		std::vector<char> buf;
 		bencode(std::back_inserter(buf), ses_state);
 		lazy_entry e;
-		lazy_bdecode(&buf[0], &buf[0] + buf.size(), e);
+		int ret = lazy_bdecode(&buf[0], &buf[0] + buf.size(), e);
+		TORRENT_ASSERT(ret == 0);
 		session_impl::mutex_t::scoped_lock l(m_impl->m_mutex);
 		m_impl->load_state(e);
 	}
@@ -478,6 +479,7 @@ namespace libtorrent
 
 	torrent_handle session::add_torrent(add_torrent_params const& params, error_code& ec)
 	{
+		ec.clear();
 		session_impl::mutex_t::scoped_lock l(m_impl->m_mutex);
 		return m_impl->add_torrent(params, ec);
 	}
@@ -877,7 +879,7 @@ namespace libtorrent
 		return m_impl->wait_for_alert(max_wait);
 	}
 
-	void session::set_alert_mask(int m)
+	void session::set_alert_mask(boost::uint32_t m)
 	{
 		session_impl::mutex_t::scoped_lock l(m_impl->m_mutex);
 		m_impl->set_alert_mask(m);
@@ -890,6 +892,9 @@ namespace libtorrent
 	}
 
 #ifndef TORRENT_NO_DEPRECATE
+	void session::set_alert_mask(int m)
+	{ set_alert_mask(boost::uint32_t(m)); }
+
 	void session::set_severity_level(alert::severity_t s)
 	{
 		session_impl::mutex_t::scoped_lock l(m_impl->m_mutex);
