@@ -202,6 +202,10 @@ void udp_socket::on_read(udp::socket* s, error_code const& e, std::size_t bytes_
 			&& e != asio::error::connection_refused
 			&& e != asio::error::connection_aborted
 			&& e != asio::error::operation_aborted
+#ifdef WIN32
+			// ERROR_MORE_DATA means the same thing as EMSGSIZE
+			&& e != error_code(ERROR_MORE_DATA, get_system_category())
+#endif
 			&& e != asio::error::message_size)
 		{
 			maybe_clear_callback(l);
