@@ -182,8 +182,22 @@ namespace libtorrent
 		{
 			// handle .torrent files that don't include the filename in the url
 			if (m_path.empty()) m_path += "/" + t->torrent_file().name();
-			else if (m_path[m_path.size() - 1] == '/') m_path += t->torrent_file().name();
-			if (!m_url.empty() && m_url[m_url.size() - 1] == '/') m_url += t->torrent_file().name();
+			else if (m_path[m_path.size() - 1] == '/')
+			{
+				std::string tmp = t->torrent_file().files().at(0).path.string();
+#ifdef TORRENT_WINDOWS
+				convert_path_to_posix(tmp);
+#endif
+				m_path += tmp;
+			}
+			else if (!m_url.empty() && m_url[m_url.size() - 1] == '/')
+			{
+				std::string tmp = t->torrent_file().files().at(0).path.string();
+#ifdef TORRENT_WINDOWS
+				convert_path_to_posix(tmp);
+#endif
+				m_url += tmp;
+			}
 		}
 
 		torrent_info const& info = t->torrent_file();
