@@ -71,8 +71,12 @@ void test_rate()
 	boost::tie(tor1, tor2, ignore) = setup_transfer(&ses1, &ses2, 0
 		, true, false, true, "_transfer", 0, &t);
 
-	ses1.set_alert_mask(alert::all_categories & ~(alert::performance_warning));
-	ses2.set_alert_mask(alert::all_categories & ~(alert::performance_warning));
+	ses1.set_alert_mask(alert::all_categories
+		& ~alert::progress_notification
+		& ~alert::stats_notification);
+	ses2.set_alert_mask(alert::all_categories
+		& ~alert::progress_notification
+		& ~alert::stats_notification);
 
 	ptime start = time_now();
 
@@ -233,8 +237,8 @@ void test_transfer(bool test_disk_full = false, bool test_allowed_fast = false)
 
 #ifndef TORRENT_DISABLE_ENCRYPTION
 	pe_settings pes;
-	pes.out_enc_policy = pe_settings::forced;
-	pes.in_enc_policy = pe_settings::forced;
+	pes.out_enc_policy = pe_settings::disabled;
+	pes.in_enc_policy = pe_settings::disabled;
 	ses1.set_pe_settings(pes);
 	ses2.set_pe_settings(pes);
 #endif
@@ -267,9 +271,13 @@ void test_transfer(bool test_disk_full = false, bool test_allowed_fast = false)
 	std::copy(priorities.begin(), priorities.end(), std::ostream_iterator<int>(std::cerr, ", "));
 	std::cerr << std::endl;
 
-	ses1.set_alert_mask(alert::all_categories & ~alert::progress_notification);
-	ses2.set_alert_mask(alert::all_categories & ~alert::progress_notification);
-	ses1.set_alert_dispatch(&print_alert);
+	ses1.set_alert_mask(alert::all_categories
+		& ~alert::progress_notification
+		& ~alert::stats_notification);
+	ses2.set_alert_mask(alert::all_categories
+		& ~alert::progress_notification
+		& ~alert::stats_notification);
+//	ses1.set_alert_dispatch(&print_alert);
 
 	ses2.set_download_rate_limit(tor2.get_torrent_info().piece_length() / 2);
 
@@ -355,13 +363,19 @@ void test_transfer(bool test_disk_full = false, bool test_allowed_fast = false)
 
 	tor2.pause();
 	alert const* a = ses2.wait_for_alert(seconds(10));
+	bool got_paused_alert = false;
 	while (a)
 	{
 		std::auto_ptr<alert> holder = ses2.pop_alert();
 		std::cerr << "ses2: " << a->message() << std::endl;
-		if (dynamic_cast<torrent_paused_alert const*>(a)) break;	
+		if (alert_cast<torrent_paused_alert>(a))
+		{
+			got_paused_alert = true;
+			break;	
+		}
 		a = ses2.wait_for_alert(seconds(10));
 	}
+	TEST_CHECK(got_paused_alert);	
 
 	std::vector<announce_entry> tr;
 	tr.push_back(announce_entry("http://test.com/announce"));
@@ -376,14 +390,15 @@ void test_transfer(bool test_disk_full = false, bool test_allowed_fast = false)
 	{
 		std::auto_ptr<alert> holder = ses2.pop_alert();
 		std::cerr << "ses2: " << a->message() << std::endl;
-		if (dynamic_cast<save_resume_data_alert const*>(a))
+		if (alert_cast<save_resume_data_alert>(a))
 		{
 			bencode(std::back_inserter(resume_data)
-				, *dynamic_cast<save_resume_data_alert const*>(a)->resume_data);
+				, *alert_cast<save_resume_data_alert>(a)->resume_data);
 			break;
 		}
 		a = ses2.wait_for_alert(seconds(10));
 	}
+	TEST_CHECK(resume_data.size());	
 
 	std::cerr << "saved resume data" << std::endl;
 
@@ -399,7 +414,9 @@ void test_transfer(bool test_disk_full = false, bool test_allowed_fast = false)
 	p.save_path = "./tmp2_transfer_moved";
 	p.resume_data = &resume_data;
 	tor2 = ses2.add_torrent(p);
-	ses2.set_alert_mask(alert::all_categories & ~alert::progress_notification);
+	ses2.set_alert_mask(alert::all_categories
+		& ~alert::progress_notification
+		& ~alert::stats_notification);
 	tor2.prioritize_pieces(priorities);
 	std::cout << "resetting priorities" << std::endl;
 	tor2.resume();
